import os
import collections
import logging

import langdetect

class Settings:
    LOGGING_FORMAT = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
    UNKNOWN_LANGUAGE = 'xx'
    SEP = '.:'
    ENDS = ['\n', '\r', '\r\n', ':']


settings = Settings()

logging.basicConfig(
    format = settings.LOGGING_FORMAT,
    level = logging.INFO
)
logger = logging.getLogger(__name__)

LANGUAGE_CODES = os.listdir(langdetect.PROFILES_DIRECTORY)
SHORTER_CODES = {'zh-cn': 'cn', 'zh-tw': 'cn'}

def detect_language(text, max_length=2):
    """ Make sure we return max_length[-letter] keys for languages"""
    code = langdetect.detect(text)
    short_code = SHORTER_CODES.get(code) if len(code) > max_length else code
    return short_code[:max_length]

def convert(LanguagesDict, sep=settings.SEP, ends=settings.ENDS, title=False):
    text_md = ''

    if isinstance(LanguagesDict, dict):

        for lang in LanguagesDict.keys():

            text = LanguagesDict.get(lang)

            if title:
                end = ends[-1]
            else:
                end = ends[0]

            text_md += '{sep}{lang}{end}{text}'.format(
                sep = sep,
                lang = lang,
                end = end,
                text = text.lstrip(),
            )

        while text_md[:2] in ends:
            text_md = text_md[2:]

        return text_md.strip()

    else:
        return LanguagesDict

def split(text, sep=settings.SEP, ends=settings.ENDS, min_key_length=2, max_key_length=2,
          autodetect=True, pargraph_sep='\n\n', markdown=False, title=False):
    """
    Splits text by `sep`, and combines texts with same keys before `ends`,
    if they are not shorter/longer than `min_key_length` and `max_key_length`.
    Assigns the rest of the parts to key called settings.UNKNOWN_LANGUAGE.
    Returns a dict.

    Detects language if not present, treating each paragraph separately.

    Tip:
         Change 'markdown' to True to get result combined back to markdown.
         Pass title=True to convert to title version, using the ':' as end.
    """
    if not text:
        return text
    # Fix: exception, if provided something like '.:en' without end symbol:
    elif len(text.split('\n')) == 1 \
        and text.startswith(sep) \
        and len(text) < len(sep)+max_key_length+min(
            [len(end) for end in ends]):
        return collections.OrderedDict(
            [(text[len(sep):len(sep)+max_key_length],
              text[len(sep)+max_key_length:])])

    result = collections.defaultdict(str)
    lang_seq = []

    for token in text.split(sep):
        if not token:
            continue

        name = settings.UNKNOWN_LANGUAGE
        chunk = token

        if len(token[:max_key_length+1]) == max_key_length+1:

            for symbol in ends:
                pos = token[:max_key_length+1].find(symbol)

                if min_key_length <= pos <= max_key_length:
                    name, chunk = token[:pos], token[pos+1:]

        if name is not settings.UNKNOWN_LANGUAGE:

            result[name] += chunk

            if name not in lang_seq:
                lang_seq.append(name)

        else:
            if autodetect:

                paragraphs = chunk.split(pargraph_sep)
                number_of_paragraphs = len(paragraphs)
                for i, paragraph in enumerate(paragraphs):
                    if not paragraph:
                        continue

                    try:
                        name = detect_language(paragraph)
                        result[name] += paragraph

                    except Exception as e:
                        result[settings.UNKNOWN_LANGUAGE] += paragraph
                        logger.info('Language not detected: {}'.format(paragraph))

                    if i < number_of_paragraphs - 1:
                        result[name] += pargraph_sep

                    if name not in lang_seq:
                        lang_seq.append(name)

            else:
                result[name] += chunk[len(name)+1:]

                if name not in lang_seq:
                    lang_seq.append(name)

    result = collections.OrderedDict(
        [(lang, result[lang]) for lang in lang_seq]
    )


    if markdown:
        return convert(result, sep=sep, ends=ends, title=title)

    return result



# -*- coding: utf-8 -*-
try:
    from urllib.request import Request, urlopen
except ImportError:
    from urllib2 import Request, urlopen

try:
    import requests
except:
    pass

def translate(text_to_translate, to_language='auto', from_langage='auto', use_requests=False):
    # Credits: https://github.com/softvar/translatr/blob/master/app/__init__.py#L22
    agents = {'User-Agent':"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1; .NET CLR 1.1.4322; .NET CLR 2.0.50727; .NET CLR 3.0.04506.30)"}
    before_trans = 'class="t0">'
    link = "http://translate.google.com/m?hl=%s&sl=%s&q=%s" % (to_language, from_langage, text_to_translate.replace(" ", "+"))

    if use_requests:
        request = requests.get(link, headers=agents)
        page = request.text
    else:
        request = Request(link, headers=agents)
        page = urlopen(request).read().decode('utf-8')

    result = page[page.find(before_trans) + len(before_trans):]
    result = result.split("<")[0]

    if result == 'html>':
        if '<div class="result-container">' in page:
            result = page.split('<div class="result-container">', 1)[-1].split('</div>')[0]
        else:
            result = None

    return result

from langsplit import splitter

def post_translate(value):
    return value

def append_machine_translations(text, langs, intext=False, use_requests=False, update_existing=False):
    ''' e.g.: langs = ['en', 'cn', 'lt'] '''
    split = splitter.split(text)
    olang = next(iter(split))
    first = split[olang]

    if intext:
        if not first.endswith('\n'):
            split[olang] += '\n'

    for lang in langs:
        if lang == olang:
            continue
        glang = lang
        if glang == 'cn':
            glang = 'zh'

        if lang.upper() not in split or update_existing:
            translation = post_translate(translate(first, glang, use_requests=use_requests))
            if translation is not None:
                split[lang.upper()] = translation.replace('  ', '\n\n')
                if intext:
                    split[lang.upper()] = split[lang.upper()].strip() + '\n\n'

    if intext:
        return splitter.convert(split)

    return split




from setuptools import find_packages, setup

setup(
    name='langsplit',
    version='0.2.5',
    description='Splits text by separator, and limited length language names, combines split text chunks into OrderedDict, or combines back to text. Supposed for extending markdown with tags for human languages.',
    url='https://github.com/mindey/langsplit',
    author='Mindey',
    author_email='mindey@qq.com',
    license='UNLICENSE',
    packages = find_packages(exclude=['docs', 'tests*']),
    install_requires=["langdetect"],
    extras_require = {
        'test': ['coverage', 'pytest', 'pytest-cov'],
    },
    zip_safe=False
)
